import { writeFile } from 'fs/promises';
import semver from 'semver';
import createDebugMessages from 'debug';
/**
 * Syncronizes dependencies between {@link SyncDependenciesParam.deepJsonFilePath} and {@link SyncDependenciesParam.packageJsonFilePath}
 */
export async function syncDependencies(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:syncDependencies');
    debug({ param });
    const { deepJsonFilePath, packageJsonFilePath: packageJsonPath, } = param;
    const { default: deepJson } = await import(deepJsonFilePath, { assert: { type: 'json' } });
    debug({ deepJson });
    const { default: packageJson } = await import(packageJsonPath, { assert: { type: 'json' } });
    debug({ packageJson });
    if (!packageJson.dependencies) {
        packageJson.dependencies = {};
    }
    const packageJsonMissingDependenciesFromDeepJson = await getPackageJsonMissingDependenciesFromDeepJson({
        deepJson,
        packageJson
    });
    debug({ packageJsonMissingDependenciesFromDeepJson });
    packageJsonMissingDependenciesFromDeepJson.forEach((dependency) => {
        packageJson.dependencies = { ...packageJson.dependencies, [dependency.name]: `~${dependency.version}` };
    });
    debug({ packageJsonDependenciesAfterAddingMissingDependencies: packageJson.dependencies });
    const syncDependenciesBasedOnDeepJsonResult = await syncDependenciesBasedOnDeepJson({
        deepJson,
        packageJson
    });
    debug({ syncDependenciesBasedOnDeepJsonResult });
    deepJson.dependencies = syncDependenciesBasedOnDeepJsonResult.deepJsonDependencies;
    packageJson.dependencies = syncDependenciesBasedOnDeepJsonResult.packageJsonDependencies;
    debug({ deepJsonDependenciesAfterMergingWithSyncDependenciesBasedOnDeepJsonResult: deepJson.dependencies });
    debug({ packageJsonDependenciesAfterMergingWithSyncDependenciesBasedOnDeepJsonResult: packageJson.dependencies });
    const syncDependenciesBasedOnPackageJsonResult = await syncDependenciesBasedOnPackageJson({
        deepJson,
        packageJson
    });
    debug({ syncDependenciesBasedOnPackageJsonResult });
    deepJson.dependencies = syncDependenciesBasedOnPackageJsonResult.deepJsonDependencies;
    packageJson.dependencies = syncDependenciesBasedOnPackageJsonResult.packageJsonDependencies;
    debug({ deepJsonDependenciesAfterMergingWithSyncDependenciesBasedOnPackageJsonResult: deepJson.dependencies });
    debug({ packageJsonDependenciesAfterMergingWithSyncDependenciesBasedOnPackageJsonResult: packageJson.dependencies });
    debug({ resultDeepJsonDependencies: deepJson.dependencies });
    debug({ resultPackageJsonDependencies: packageJson.dependencies });
    writeFile(deepJsonFilePath, JSON.stringify(deepJson, null, 2));
    writeFile(packageJsonPath, JSON.stringify(packageJson, null, 2));
}
async function syncDependenciesBasedOnDeepJson(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:syncDependenciesBasedOnDeepJson');
    debug({ param });
    const { deepJson, packageJson } = param;
    let packageJsonDependencies = packageJson.dependencies;
    debug({ packageJsonDependencies });
    let deepJsonDependencies = deepJson.dependencies;
    debug({ deepJsonDependencies });
    deepJson.dependencies.forEach((dependency, index) => {
        debug({ dependency, index });
        const deepJsonDependencyVersionWithoutRange = semver.minVersion(dependency.version)?.version;
        debug({ dependencyVersionWithoutRange: deepJsonDependencyVersionWithoutRange });
        if (!deepJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const packageJsonDependencyVersionWithoutRange = semver.minVersion(packageJson.dependencies[dependency.name])?.version;
        debug({ packageJsonDependencyVersionWithoutRange });
        if (!packageJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const isDeepJsonVersionGreater = semver.gt(deepJsonDependencyVersionWithoutRange, packageJsonDependencyVersionWithoutRange);
        if (isDeepJsonVersionGreater) {
            packageJsonDependencies[dependency.name] = `~${deepJsonDependencyVersionWithoutRange}`;
        }
        else {
            deepJsonDependencies[index] = {
                name: dependency.name,
                version: deepJsonDependencyVersionWithoutRange
            };
        }
    });
    const result = { packageJsonDependencies, deepJsonDependencies };
    debug({ result });
    return result;
}
async function syncDependenciesBasedOnPackageJson(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:syncDependenciesBasedOnPackageJson');
    debug({ param });
    const { deepJson, packageJson } = param;
    let packageJsonDependencies = packageJson.dependencies;
    debug({ packageJsonDependencies });
    let deepJsonDependencies = deepJson.dependencies;
    debug({ deepJsonDependencies });
    Object.entries(packageJsonDependencies).forEach(([dependencyName, dependencyVersion]) => {
        debug({ dependencyName, dependencyVersion });
        const packageJsonDependencyVersionWithoutRange = semver.minVersion(dependencyVersion)?.version;
        debug({ dependencyVersionWithoutRange: packageJsonDependencyVersionWithoutRange });
        if (!packageJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const deepJsonDependencyIndex = deepJsonDependencies.findIndex(dependency => dependency.name === dependencyName);
        debug({ deepJsonDependencyIndex });
        if (deepJsonDependencyIndex === -1)
            return;
        const deepJsonDependency = deepJsonDependencies[deepJsonDependencyIndex];
        debug({ deepJsonDependency });
        const deepJsonDependencyVersionWithoutRange = semver.minVersion(deepJsonDependency.version)?.version;
        debug({ deepJsonDependencyVersionWithoutRange });
        if (!deepJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const isPackageJsonVersionGreater = semver.gt(packageJsonDependencyVersionWithoutRange, deepJsonDependencyVersionWithoutRange);
        if (isPackageJsonVersionGreater) {
            deepJsonDependencies[deepJsonDependencyIndex] = {
                name: dependencyName,
                version: packageJsonDependencyVersionWithoutRange
            };
        }
        else {
            packageJsonDependencies[dependencyName] = `~${deepJsonDependencyVersionWithoutRange}`;
        }
    });
    const result = { packageJsonDependencies, deepJsonDependencies };
    debug({ result });
    return result;
}
async function getPackageJsonMissingDependenciesFromDeepJson(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:getPackageJsonMissingDependenciesFromDeepJson');
    const { deepJson, packageJson } = param;
    const missingDependenciesFromDeepJson = deepJson.dependencies.filter((dependency) => packageJson.dependencies[dependency.name] === undefined);
    debug({ missingDependenciesFromDeepJson });
    return missingDependenciesFromDeepJson;
}
